# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Support for py.test framework."""

# Local imports
from spyder_unittest.backend.runnerbase import RunnerBase


class PyTestRunner(RunnerBase):
    """Class for running tests within py.test framework."""

    module = 'pytest'
    name = 'py.test'

    def create_argument_list(self):
        """Create argument list for testing process (dummy)."""
        return ['--junit-xml', self.resultfilename]



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for frameworkregistry.py"""

# Third party imports
import pytest

# Local imports
from spyder_unittest.backend.frameworkregistry import FrameworkRegistry


class MockRunner:
    name = 'foo'

    def __init__(self, *args):
        self.init_args = args


def test_frameworkregistry_when_empty():
    reg = FrameworkRegistry()
    with pytest.raises(KeyError):
        reg.create_runner('foo', None, 'temp.txt')


def test_frameworkregistry_after_registering():
    reg = FrameworkRegistry()
    reg.register(MockRunner)
    runner = reg.create_runner('foo', None, 'temp.txt')
    assert isinstance(runner, MockRunner)
    assert runner.init_args == (None, 'temp.txt')



# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Support for Nose framework."""

# Local imports
from spyder_unittest.backend.runnerbase import RunnerBase


class NoseRunner(RunnerBase):
    """Class for running tests within Nose framework."""

    module = 'nose'
    name = 'nose'

    def create_argument_list(self):
        """Create argument list for testing process."""
        return ['--with-xunit', '--xunit-file={}'.format(self.resultfilename)]



# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for unittestgui.py."""

# Standard library imports
import os

# Third party imports
from pytestqt import qtbot
from qtpy.QtCore import Qt
import pytest

# Local imports
from spyder_unittest.backend.runnerbase import Category, TestResult
from spyder_unittest.widgets.configdialog import Config
from spyder_unittest.widgets.unittestgui import (UnitTestDataTree,
                                                 UnitTestWidget)

try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock  # Python 2


def test_unittestdatatree_shows_short_name_in_table(qtbot):
    datatree = UnitTestDataTree()
    res = TestResult(Category.OK, 'status', 'bar', 'foo', '', 0, '')
    datatree.testresults = [res]
    datatree.populate_tree()
    assert datatree.topLevelItem(0).data(1, Qt.DisplayRole) == 'bar'


def test_unittestdatatree_shows_full_name_in_tooltip(qtbot):
    datatree = UnitTestDataTree()
    res = TestResult(Category.OK, 'status', 'bar', 'foo', '', 0, '')
    datatree.testresults = [res]
    datatree.populate_tree()
    assert datatree.topLevelItem(0).data(1, Qt.ToolTipRole) == 'foo.bar'


@pytest.mark.parametrize('framework', ['py.test', 'nose'])
def test_run_tests_and_display_results(qtbot, tmpdir, monkeypatch, framework):
    """Basic check."""
    os.chdir(tmpdir.strpath)
    testfilename = tmpdir.join('test_foo.py').strpath

    with open(testfilename, 'w') as f:
        f.write("def test_ok(): assert 1+1 == 2\n"
                "def test_fail(): assert 1+1 == 3\n")

    MockQMessageBox = Mock()
    monkeypatch.setattr('spyder_unittest.widgets.unittestgui.QMessageBox',
                        MockQMessageBox)

    widget = UnitTestWidget(None)
    qtbot.addWidget(widget)
    config = Config(wdir=tmpdir.strpath, framework=framework)
    with qtbot.waitSignal(widget.sig_finished, timeout=10000, raising=True):
        widget.run_tests(config)

    MockQMessageBox.assert_not_called()
    dt = widget.datatree
    itemcount = dt.topLevelItemCount()
    assert itemcount == 2
    assert dt.topLevelItem(0).data(0, Qt.DisplayRole) == 'ok'
    assert dt.topLevelItem(0).data(1, Qt.DisplayRole) == 'test_ok'
    assert dt.topLevelItem(0).data(1, Qt.ToolTipRole) == 'test_foo.test_ok'
    assert dt.topLevelItem(0).data(2, Qt.DisplayRole) == ''
    assert dt.topLevelItem(1).data(0, Qt.DisplayRole) == 'failure'
    assert dt.topLevelItem(1).data(1, Qt.DisplayRole) == 'test_fail'
    assert dt.topLevelItem(1).data(1, Qt.ToolTipRole) == 'test_foo.test_fail'


def test_run_tests_using_unittest_and_display_results(qtbot, tmpdir,
                                                      monkeypatch):
    """Basic check."""
    os.chdir(tmpdir.strpath)
    testfilename = tmpdir.join('test_foo.py').strpath

    with open(testfilename, 'w') as f:
        f.write("import unittest\n"
                "class MyTest(unittest.TestCase):\n"
                "   def test_ok(self): self.assertEqual(1+1, 2)\n"
                "   def test_fail(self): self.assertEqual(1+1, 3)\n")

    MockQMessageBox = Mock()
    monkeypatch.setattr('spyder_unittest.widgets.unittestgui.QMessageBox',
                        MockQMessageBox)

    widget = UnitTestWidget(None)
    qtbot.addWidget(widget)
    config = Config(wdir=tmpdir.strpath, framework='unittest')
    with qtbot.waitSignal(widget.sig_finished, timeout=10000, raising=True):
        widget.run_tests(config)

    MockQMessageBox.assert_not_called()
    dt = widget.datatree
    assert dt.topLevelItemCount() == 2
    assert dt.topLevelItem(0).data(0, Qt.DisplayRole) == 'FAIL'
    assert dt.topLevelItem(0).data(1, Qt.DisplayRole) == 'test_fail'
    assert (dt.topLevelItem(0).data(1, Qt.ToolTipRole) ==
            'test_foo.MyTest.test_fail')
    assert dt.topLevelItem(0).data(2, Qt.DisplayRole) == ''
    assert dt.topLevelItem(1).data(0, Qt.DisplayRole) == 'ok'
    assert dt.topLevelItem(1).data(1, Qt.DisplayRole) == 'test_ok'
    assert (dt.topLevelItem(1).data(1, Qt.ToolTipRole) ==
            'test_foo.MyTest.test_ok')
    assert dt.topLevelItem(1).data(2, Qt.DisplayRole) == ''



# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for baserunner.py"""

# Local imports
from spyder_unittest.backend.runnerbase import Category, RunnerBase


def test_runnerbase_with_nonexisting_module():
    class FooRunner(RunnerBase):
        module = 'nonexisiting'

    assert not FooRunner.is_installed()


def test_runnerbase_load_data(tmpdir):
    result_file = tmpdir.join('results')
    result_txt = """<?xml version="1.0" encoding="utf-8"?>
<testsuite errors="0" failures="1" name="pytest" skips="1" tests="3" time="0.1">
<testcase classname="test_foo" file="test_foo.py" line="2" name="test1" time="0.04"></testcase>
<testcase classname="test_foo" file="test_foo.py" line="5" name="test2" time="0.01">
    <failure message="failure message">text</failure>
</testcase>
<testcase classname="test_foo" file="test_foo.py" line="8" name="test3" time="0.05">
    <skipped message="skip message">text2</skipped>
</testcase></testsuite>"""
    result_file.write(result_txt)
    runner = RunnerBase(None, result_file.strpath)
    results = runner.load_data()
    assert len(results) == 3

    assert results[0].category == Category.OK
    assert results[0].status == 'ok'
    assert results[0].name == 'test1'
    assert results[0].module == 'test_foo'
    assert results[0].message == ''
    assert results[0].time == 0.04
    assert results[0].extra_text == ''

    assert results[1].category == Category.FAIL
    assert results[1].status == 'failure'
    assert results[1].name == 'test2'
    assert results[1].module == 'test_foo'
    assert results[1].message == 'failure message'
    assert results[1].time == 0.01
    assert results[1].extra_text == 'text'

    assert results[2].category == Category.SKIP
    assert results[2].status == 'skipped'
    assert results[2].name == 'test3'
    assert results[2].module == 'test_foo'
    assert results[2].message == 'skip message'
    assert results[2].time == 0.05
    assert results[2].extra_text == 'text2'


def test_runnerbase_load_data_failing_test_with_stdout(tmpdir):
    result_file = tmpdir.join('results')
    result_txt = """<?xml version="1.0" encoding="utf-8"?>
<testsuite errors="0" failures="1" name="pytest" skips="0" tests="1" time="0.1">
<testcase classname="test_foo" file="test_foo.py" line="2" name="test1" time="0.04">
<failure message="failure message">text</failure>
<system-out>stdout text
</system-out></testcase></testsuite>"""
    result_file.write(result_txt)
    runner = RunnerBase(None, result_file.strpath)
    results = runner.load_data()
    assert results[0].extra_text == (
        'text\n\n' + '----- Captured stdout -----\n' + 'stdout text')


def test_runnerbase_load_data_passing_test_with_stdout(tmpdir):
    result_file = tmpdir.join('results')
    result_txt = """<?xml version="1.0" encoding="utf-8"?>
<testsuite errors="0" failures="0" name="pytest" skips="0" tests="1" time="0.1">
<testcase classname="test_foo" file="test_foo.py" line="2" name="test1" time="0.04">
<system-out>stdout text
</system-out></testcase></testsuite>"""
    result_file.write(result_txt)
    runner = RunnerBase(None, result_file.strpath)
    results = runner.load_data()
    assert results[0].extra_text == (
        '----- Captured stdout -----\n' + 'stdout text')
